@@ -3,7 +3,7 @@
 ;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
 ;;   2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 
-;; Copyright (C) 2010  Gaétan Allaert
+;; Copyright (C) 2010, 2011  Gaétan Allaert
 
 ;; Author: Markus Heritsch <Markus.Heritsch@studbox.uni-stuttgart.de>
 ;;      Rolf Ebert <ebert@inf.enst.fr>
@@ -544,7 +544,8 @@ All the directories are returned as absolute directories."
      (ada-get-absolute-dir-list (ada-xref-get-project-field 'obj_dir)
 				build-dir)
 
-     (unless in-spark-annotation
+     (if in-spark-annotation
+	 (list (spark-output-dir))
        ;; Add the standard runtime at the end
        ada-xref-runtime-library-ali-path))))
 
@@ -1007,7 +1008,7 @@ If LOCAL-ONLY is non-nil, only declarations in the current file are returned."
   (ada-require-project-file)
 
   (let* ((identlist (ada-read-identifier pos))
-	 (in-spark-annotation (ada-in-spark-annotation-p))
+	 (in-spark-annotation (spark-in-annotation-p))
 	 (alifile (ada-get-ali-file-name (ada-file-of identlist) nil))
 	 (slifile nil)
 	 (process-environment (ada-set-environment)))
@@ -1056,7 +1057,7 @@ If ARG is non-nil, the contents of the old *gnatfind* buffer is preserved."
   (cond ((eq ada-gnatfind-or-gps 'gnatfind)
 	 (ada-find-any-references-gnatfind entity file line column sli-line sli-column local-only append))
 	((eq ada-gnatfind-or-gps 'gps)
-	 (ada-find-any-references-gps entity file line column sli-line sli-column local-only append))))
+	 (ada-find-any-references-gps      entity file line column sli-line sli-column local-only append))))
 
 (defun ada-find-any-references-gnatfind
   (entity &optional file line column sli-line sli-column local-only append)
@@ -1081,7 +1082,7 @@ existing buffer `*gnatfind*', if there is one."
 	 (switches (ada-xref-get-project-field 'gnatfind_opt))
 	 ;; FIXME: use gpr_file
 	 (cross-prefix (ada-xref-get-project-field 'cross_prefix))
-	 (in-spark-annotation (ada-in-spark-annotation-p))
+	 (in-spark-annotation (spark-in-annotation-p))
 	 (command (concat cross-prefix "gnat find " switches " "
 			  quote-entity
 			  (if (and file (not in-spark-annotation)) (concat ":" (file-name-nondirectory file)))
@@ -1242,7 +1243,7 @@ existing buffer `*gnatfind*', if there is one."
   ;; Initialize function-local variables and jump to the .ali/.sli buffer
   ;; Note that for regexp search is case insensitive too
   (let* ((curbuf (current-buffer))
-	 (in-spark-annotation (ada-in-spark-annotation-p))
+	 (in-spark-annotation (spark-in-annotation-p))
 	 (identlist (ada-read-identifier pos))
 	 (sofar (concat "^[0-9]+[a-zA-Z][0-9]+[ *]\\("
 			(regexp-quote (ada-name-of identlist))
@@ -1323,7 +1324,7 @@ If OTHER-FRAME is non-nil, display the cross-reference in another frame."
   ;;  First try the standard algorithm by looking into the .ali/.sli file, but if
   ;;  that file was too old or even did not exist, try to look in the whole
   ;;  object path for a possible location.
-  (let ((in-spark-annotation (ada-in-spark-annotation-p))
+  (let ((in-spark-annotation (spark-in-annotation-p))
 	(identlist (ada-read-identifier pos)))
     (condition-case err
 	(ada-find-in-ali identlist other-frame in-spark-annotation)
@@ -1903,7 +1904,7 @@ macros `ada-name-of', `ada-line-of', `ada-column-of', `ada-file-of',..."
 
   ;; check if it really is an identifier
   (if (and (ada-in-comment-p)
-	   (not (ada-in-spark-annotation-p)))
+	   (not (spark-in-annotation-p)))
       (error "Inside comment"))
 
   (let (identifier identlist)
@@ -2436,7 +2437,7 @@ Return the position of the declaration in the buffer, or nil if not found."
 		  (goto-char (match-end 0))
 		  (looking-at "_")))
 	   (or (not (ada-in-string-or-comment-p))
-	       (ada-in-spark-annotation-p))
+	       (spark-in-annotation-p))
 	   (or
 	    ;; variable declaration ?
 	    (save-excursion
