@@ -1,7 +1,7 @@
 ;; spark-fix-error.el --- utilities for automatically fixing
 ;; errors reported by the SPARK Examiner.
 
-;; Copyright (C) 2010  Gaétan Allaert
+;; Copyright (C) 2010, 2011  Gaétan Allaert
 
 ;; Author: Gaétan Allaert <gaetan.allaert@belgacom.net>
 ;; Maintainer: Gaétan Allaert <gaetan.allaert@belgacom.net>
@@ -161,7 +161,7 @@
       (setq removing-states-list (cdr removing-states-list)))))
 
 ;; Fix semantic error
-(defun spark-fix-sem-error (state spark-global-fixed)
+(defun spark-fix-sem-error (state spark-own-or-global-fixed spark-global-varlist)
   (let ((begin nil)
 	(end nil)
 	(paramlist nil)
@@ -169,7 +169,7 @@
 	(paramlist-reformatted nil))
 
     ;; find start of current variable-list
-    (ada-search-ignore-string-comment "global\\>" t nil nil nil t)
+    (ada-search-ignore-string-comment "\\<own\\|global\\>" t nil nil nil t)
     (save-excursion (beginning-of-line)
 		    (ada-indent-current))
     (forward-word 1)
@@ -183,9 +183,12 @@
     (let ((p (point)))
       (while (save-excursion (and
 			      (ada-goto-next-non-ws nil nil t)
+			      (spark-in-annotation-p)
 			      (or (re-search-forward "protected[ \t]" (+ (point) 10) t)
 				  (re-search-forward "in[ \t]" (+ (point) 3) t)
-				  (re-search-forward "out[ \t]" (+ (point) 4) t))
+				  (re-search-forward "out[ \t]" (+ (point) 4) t)
+				  (save-excursion (skip-syntax-forward "w")
+						  (not (spark-after-keyword-p))))
 			      (setq p (point))))
 	(progn (goto-char p)
 	       (ada-search-ignore-string-comment ";" nil nil nil nil t))))
@@ -202,10 +205,13 @@
     (delete-region begin end)
 
     ;; fix the variable-list
-    (setq paramlist-fixed (funcall spark-global-fixed paramlist state))
+    (setq paramlist-fixed (funcall spark-own-or-global-fixed paramlist state))
 
     ;; reformat the variable-list
-    (setq paramlist-reformatted (spark-global-varlist-reformat paramlist-fixed))
+    (setq paramlist-reformatted
+	  (if (spark-own-varlist-no-type-p paramlist)
+	      (spark-own-or-global-varlist-reformat paramlist-fixed spark-global-varlist)
+	    paramlist-fixed))
 
     ;; insert the new variable-list
     (goto-char begin)
@@ -281,16 +287,16 @@
   (remove-global-state paramlist state nil t))
 
 (defun spark-add-missing-global-in-state (state)
-  (spark-fix-sem-error state 'add-missing-global-in-state))
+  (spark-fix-sem-error state 'add-missing-global-in-state t))
 
 (defun spark-add-missing-global-out-state (state)
-  (spark-fix-sem-error state 'add-missing-global-out-state))
+  (spark-fix-sem-error state 'add-missing-global-out-state t))
 
 (defun spark-remove-global-in-state (state)
-  (spark-fix-sem-error state 'remove-global-in-state))
+  (spark-fix-sem-error state 'remove-global-in-state t))
 
 (defun spark-remove-global-out-state (state)
-  (spark-fix-sem-error state 'remove-global-out-state))
+  (spark-fix-sem-error state 'remove-global-out-state t))
 
 ;; Fix
 (defun spark-fix-one-error (source-buffer error-type)
@@ -337,8 +343,7 @@
 		  (= error-type 1))
 	  (spark-remove-global-out-state state)
 	  (setq new-error-type 1))))
-     ((or (looking-at (concat "Flow Error  *3 - The previously stated dependency of the exported value of " identifier-name-regexp " on the imported value of " identifiers-name-regexp " has been omitted\."))
-	  (looking-at (concat "Flow Error  *4 - The dependency of the exported value of " identifier-name-regexp " on the imported value of " identifiers-name-regexp " has not been previously stated\."))
+     ((or (looking-at (concat "Flow Error  *4 - The dependency of the exported value of " identifier-name-regexp " on the imported value of " identifiers-name-regexp " has not been previously stated\."))
 	  (looking-at (concat "Flow Error  *601 - " identifier-name-regexp " may be derived from the imported value(s) of " identifiers-name-regexp "\.")))
       (let ((state (match-string 1))
 	    (missing-states (match-string 2)))
@@ -347,7 +352,8 @@
 		  (= error-type 2))
 	  (spark-add-missing-derive-state state missing-states)
 	  (setq new-error-type 2))))
-     ((looking-at (concat "Flow Error  *50 - " identifier-name-regexp " is not derived from the imported value(s) of " identifiers-name-regexp "\."))
+     ((or (looking-at (concat "Flow Error  *3 - The previously stated dependency of the exported value of " identifier-name-regexp " on the imported value of " identifiers-name-regexp " has been omitted\."))
+	  (looking-at (concat "Flow Error  *50 - " identifier-name-regexp " is not derived from the imported value(s) of " identifiers-name-regexp "\.")))
       (let ((state (match-string 1))
 	    (removing-states (match-string 2)))
 	(pop-to-buffer source-buffer)
@@ -363,6 +369,15 @@
 	  (delete-region (point) end)
 	  (ada-format-spark-derives)
 	  (setq new-error-type 9))))
+     ((looking-at (concat "Semantic Error  *337 - Package " identifier-name-regexp " may not be used as a prefix here\."))
+      (let ((state (match-string 1))
+	    end)
+	(pop-to-buffer source-buffer)
+	(when (= error-type 0)
+	  (setq begin (point))
+	  (looking-at (concat state "\."))
+	  (delete-region begin (match-end 0))
+	  (setq new-error-type 9))))
      ((looking-at (concat "Warning  *380 - Casing inconsistent with declaration. Expected casing is " identifier-name-regexp "\."))
       (let ((correct (match-string 1))
 	    begin)
@@ -373,7 +388,9 @@
 	  (delete-region begin (match-end 0))
 	  (insert correct)
 	  (setq new-error-type 9))))
-     )
+     ((or (looking-at (concat "Warning  *430 - SLI generation abandoned owing to syntax or semantic errors or multiple units in a single source file\."))
+	  (looking-at (concat "Warning  *399 - The called subprogram has semantic errors in its interface (parameters and/or annotations) which prevent flow analysis of this call\.")))
+      (pop-to-buffer source-buffer)))
     new-error-type))
 
 (defun spark-fix-error ()
@@ -403,7 +420,7 @@
 	    (search-forward "Flow Error " (line-end-position) t)
 	    (search-forward "Warning " (line-end-position) t))
 	(progn (beginning-of-line)
-	       (looking-at "[^:]*:[0-9]*:[0-9]*: *")
+	       (looking-at "[^:]*:[1-9][0-9]*:[1-9][0-9]*: *")
 	       (setq fix-grouped-errors (match-string 0))
 	       (beginning-of-line))
       (error "spark-fix-error: error not found from point %d" (point)))
